import time
from .Mail import Mail as Mailclass
class Inbox:
    def __init__(self, xitroo, inbox: dict = None):
        """
        Inbox constructor.
        :param xitroo: Xitroo object.
        :param inbox: optional dict of Inbox.
        :type inbox: :class:`dict`
        :type xitroo: :class:`xitroo.Xitroo.Xitroo`
        """
        self._xitroo = xitroo
        if inbox == "":
            self._inbox: dict = inbox
        else:
            self._inbox: dict = self.getRawInbox()

    def getRawInbox(self, mailsPerPage=25) -> dict:
        """
        Get raw Inbox.
        :param mailsPerPage: number of mails to fetch.
        :type mailsPerPage: :class:`int`
        :rtype: :class:`dict`
        :return: :class:`dict` of inbox
        """
        params: dict[str, str] = {
            "locale": self._xitroo._locale,
            "mailAddress": self._xitroo._mailAddress,
            "mailsPerPage": mailsPerPage,
            "minTimestamp": "0",
            "maxTimestamp": time.time()
        }
        reqeust: dict = self._xitroo._session.get(self._xitroo._MAILS, params=params).json()
        # Translate for later usage #1
        if "type" in reqeust:
            return {'totalMails': 0, 'mails': []}
        return reqeust

    def getTotalMails(self) -> int:
        """
        Get total number of mails in ``Inbox``.
        :rtype: :class:`int`
        :return: :class:`int`
        """
        return self._inbox['totalMails']

    def getRawMails(self) -> dict:
        """
        Get raw mails in ``Inbox``.
        :rtype: :class:`dict`
        :return: :class:`dict`
        """
        return self._inbox['mails']

    def getRawMail(self, index: int) -> dict:
        """
        Get raw mail in ``Inbox`` by index.
        :param index: index of mail to fetch.
        :type index: :class:`int`
        :rtype: :class:`dict`
        :return: :class:`dict`
        """
        if not index < 0 or not index >= self.getTotalMails():
            return self._inbox['mails'][index]

    def getMail(self, index: int) -> Mailclass:
        """
        Get :class:`Mailclass` object in ``Inbox`` by index.
        :param index: index of mail to fetch.
        :type index: :class:`int`
        :rtype: :class:`Mailclass`
        :return: :class:`Mailclass`
        """
        return self._xitroo.Mail(self.getRawMail(index)["_id"])

    def getMailFirst(self) -> Mailclass:
        """
        Get first mail in ``Inbox``.
        :rtype: :class:`Mailclass`
        :return: :class:`Mailclass`
        """
        return self._xitroo.Mail(self.getRawMail(0)["_id"])

    def getMailLast(self) -> Mailclass:
        """
        Get last mail in ``Inbox``.
        :rtype: :class:`Mailclass`
        :return: :class:`Mailclass`
        """
        return self._xitroo.Mail(self.getRawMail(-1)["_id"])


    # def searchRegex(self):
    #     return self._xitroo.searchInboxRegex()



import random
import re
import string
import time
import requests
from .exceptions import *
from .Mail import Mail as Mailclass
from .Inbox import Inbox as Inboxclass
from .Captcha import Captcha as Captchaclass
from .Search import SearchMail as Searchclass

class Xitroo:
    _APIURL = 'https://api.xitroo.com/v1'
    _MAILS = _APIURL + '/mails?'
    _MAIL = _APIURL + '/mail?'
    _SEND = _APIURL + '/mail/send'
    _ATTACHMENT = _APIURL + '/mail/attachment?'
    _GETCAPTCHA = _SEND + '/requestCaptcha?'
    _SENDCAPTCHA = _SEND + '/verifyCaptcha?'
    _SENDMAIL = _SEND + '/sendMail?'

    def __init__(self, mailAddress: str, header = {}, session: requests.Session = None):
        """
        Xitroo API constructor.
        :param mailAddress: mail address to check for or send mails.
        :param header: optional headers to send with request.
        :param session: optional requests.Session object.
        :type mailAddress: :class:`str`
        :type header: :class:`collections.abc.MutableMapping[str, str | bytes]`
        :type session: :class:`requests.Session`
        """
        self._l: list[str] = ["de", "fr", "com"]
        self._locale: str = mailAddress.strip().split(".")[-1]
        self._header = header
        self._mailAddress: str = mailAddress.strip()
        self._session: requests.Session = requests.Session() if session is None else session

        if self._locale not in self._l:
            raise InvalidLocale("Error Locale: mailAddress must end with any locale from list: " + ", ".join(self._l) + " (Choosen: " + mailAddress.split('.')[-1] + ")")

        if len(self._header) != 0:
            self._session.headers.update(self._header)


    def getMailAddress(self) -> str:
        """
        Returns the mail address of the current mail address.
        :return: :class:`str`
        """
        return self._mailAddress

    def setMailAddress(self, mailAddress: str) -> None:
        """
        :param mailAddress: Sets the mail address to your given argument.
        :type mailAddress: :class:`str`
        """
        mailAddress: str = mailAddress.strip()
        self._locale: str = mailAddress.split(".")[-1]
        self._mailAddress: str = mailAddress

    def getHeader(self):
        """
        returns the header of the current :class:`requests.Session` instance.
        :return: :class:`collections.abc.MutableMapping[str, str | bytes]`
        """
        return self._header

    def setHeader(self, header) -> None:
        """
        :param header: sets the header of the current :class:`requests.Session` instance.
        :type header: :class:`collections.abc.MutableMapping[str, str | bytes]`
        """
        self._header = header
        self._session.headers.update(self._header)

    def getSession(self) -> requests.Session:
        """
        returns the :class:`requests.Session` instance.
        :return: :class:`requests.Session``
        """
        return self._session

    def setSession(self, session: requests.Session) -> None:
        """
        :param session: sets the :class:`requests.Session` instance.
        :type session: :class:`requests.Session``
        """
        self._session: requests.Session = session
        self._header = self._session.headers



    def equals(self, xitroo) -> bool:
        """
        :param xitroo: :class:`Xitroo` Object to compare to.
        :type xitroo: :class:`Xitroo`
        :rtype: :class:`bool`
        :return: :class:`bool` - True if equals, false otherwise.
        """
        return self.getMailAddress == xitroo.getMailAddress and self.getHeader == xitroo.getHeader

    def copy(self) -> __init__:
        """
        Creates a new :class:`Xitroo` instance with a copy of the current :class:`Xitroo` instance.
        :rtype: :class:`Xitroo`
        :return: Copy of :class:`Xitroo` instance
        """
        return self.__class__(self.getMailAddress(), self.getHeader())

    def copyTo(self, XitrooObject: __init__) -> __init__:
        """
        Copy current :class:`Xitroo` instance to given object :class:`Xitroo` instance.
        :param XitrooObject: Already instanced :class:`Xitroo`
        :type XitrooObject: :class:`Xitroo`
        :rtype: :class:`Xitroo`
        :return: Copy of :class:`Xitroo` instance
        """
        pass


    def Mail(self, mailId: str) -> Mailclass:
        """
        Create Mail Object to read mail
        :param mailId: Identifier to read mail from.
        :type mailId: :class:`str`
        :rtype: :class:`Mailclass`
        :return: :class:`Mailclass` Object.
        """
        return Mailclass(self, mailId)

    def Inbox(self, inbox: dict = None) -> Inboxclass:
        """
        Create Inbox Object to read Inbox
        :param inbox: optional Inbox dict if there is one. Should look like this {'totalMails': 0, 'mails': []}.
        :type inbox: :class:`dict`
        :rtype: :class:`Inboxclass`
        :return: :class:`Inboxclass` Object
        """
        return Inboxclass(self, inbox)

    def Captcha(self) -> Captchaclass:
        """
        Create Captcha Object to get or verify captcha
        :rtype: :class:`Captchaclass`
        :return: :class:`Captchaclass` Object
        """
        return Captchaclass(self)

    # Rewrite Search
    def searchInbox(self) -> Searchclass:
        """
        Create Search Object to search Inbox using :class:`Searchclass`
        :rtype: :class:`Searchclass`
        :return: :class:`Searchclass` Object
        """
        return Searchclass(self)

    def getRawInbox(self, mailsPerPage=25) -> dict:
        """
        Read current Inbox from ``mailaddress`` to dict
        :param mailsPerPage: optional Number of mails to read from Inbox.
        :type mailsPerPage: :class:`int`
        :rtype: :class:`dict`
        :return: :class:`dict` of Inbox
        """
        return Inboxclass(self).getRawInbox(mailsPerPage)

    def getRawMail(self, mailId: str) -> dict:
        """
        Read current Mail from ``mailId`` to dict
        :param mailId: Identifier to read mail from.
        :type mailId: :class:`str`
        :rtype: :class:`dict`
        :return: :class:`dict` of Mail
        """
        return Mailclass(self, mailId).getRawMail()


    def _verifyCaptchaAsUserInput(self) -> str:
        id: str = ""
        solution: str = ""
        r = {"authSuccess": False}
        while(not r["authSuccess"]):
            reload: bool = True
            params: dict[str, str] = {"locale": self._locale}
            while(reload):
                r: dict = self._session.get(self._GETCAPTCHA, headers=self._header, params=params).json()
                id: str = r["authID"]
                captcha: str = r["captchaCode"]
                print(captcha)
                solution: str = input("Solve Captcha (r for reload, e for exit): ")
                if solution != "r":
                    reload = False
                if solution == "e":
                    return ""
            params.update({
                "authID": id,
                "captchaSolution": solution
            })
            r: dict = self._session.get(self._SENDCAPTCHA, headers=self._header, params=params).json()
            if not r["authSuccess"]:
                print("Captcha failed")
        return id

    def sendMail(self, recipient: str, subject: str, Text: str, mode: int = 1, id: str = "") -> bool:
        """
        Send Mail as given ``mailaddress`` in constructor to ``recipient`` with given ``subject`` and ``Text``.
        :param recipient: recipient email address.
        :type recipient: :class:`str`
        :param subject: subject to send with.
        :type subject: :class:`str`
        :param Text: text to send.
        :type Text: :class:`str`
        :param mode: optional send mode. - **1**: manual userinput to get captcha id; **0**: no userinput, but requires captcha id. Create captcha id with ``Xitroo.Captcha`` :class:`Captchaclass`.
        :type mode: :class:`int`
        :param id: optional captcha id which is required to send mail if mode 0 is selected.
        :type id: :class:`str`
        :rtype: :class:`bool`
        :return: :class:`bool` - True if sent successfully, false otherwise.
        """
        if mode:
            id: str = self._verifyCaptchaAsUserInput()
        params: dict[str, str] = {"locale": self._locale}
        if not id:
            return False
        data: dict[str, str] = {
            "authID": id,
            "bodyText": Text,
            "from": self._mailAddress,
            "recipient": recipient,
            "replyMailID": "",
            "subject": subject
        }
        r: requests.Response = self._session.post(self._SENDMAIL, headers=self._header, params=params, data=data)
        if r.status_code != 200:
            return False
        return True

    @staticmethod
    def generate(prefix: str = "", suffix: str = "", locale: str = "de", randomletterscount: int = 10) -> str:
        """
        Static Method to Generate random email String of given ``length``, ``prefix``, ``suffix`` and ``locale``.
        :param prefix: optional prefix to generate email String.
        :type prefix: :class:`str`
        :param suffix: optional suffix to generate email String.
        :type suffix: :class:`str`
        :param locale: optional locale to generate domain of email string. [de, fr, com] only are **supported**.
        :type locale: :class:`str`
        :param randomletterscount: optional number of random letters to generate into email String.
        :type randomletterscount: :class:`int`
        :rtype: :class:`str`
        :return: email String.
        """
        return prefix + "".join(random.choices(string.ascii_letters, k=randomletterscount)) + suffix + "@xitroo." + locale

    @staticmethod
    def getCode(body: str, codelength: int = 6) -> str:
        """
        Static Method to get code from given ``body`` and ``codelength`` to identify the code.
        :param body: bodytext of email.
        :param codelength: optional length of code to parse.
        :type body: :class:`str`
        :type codelength: :class:`int`
        :rtype: :class:`str`
        :return: Verification Code.
        """
        return re.search('\\d{' + str(codelength) + '}', body).group(0)

    def getLatestMail(self) -> Mailclass | None:
        """
        Get latest Mail from ``mailaddress``
        :rtype: :class:`Mailclass` | None
        :return: :class:`Mailclass` Object of latest Mail or None if no latest Mail is found.
        """
        if self.Inbox().getTotalMails() != 0:
            return self.Inbox().getMailFirst()
        return None

    def waitForLatestMail(self, maxTime=60, sleepTime=5, checkMail=100) -> Mailclass:
        """
        Wait until latest Mail.
        :param maxTime: optional maximum time of the difference between current time and latest Mail received mail time to get only new mail.
        :param sleepTime: optional sleep time in seconds between checking latest mail.
        :param checkMail: optional Interval to check if latest Mail is found.
        :type maxTime: :class:`int`
        :type sleepTime: :class:`int`
        :type checkMail: :class:`int`
        :rtype: :class:`Mailclass`
        :return: :class:`Mailclass` Object of latest Mail.
        """
        for i in range(checkMail):
            latest = self.getLatestMail()
            if not(latest.getRawMail()["arrivalTimestamp"] + maxTime < time.time()):
                return latest
            print("Waiting for latest mail...")
            time.sleep(sleepTime)



import base64
import requests
import os

class Mail:
    def __init__(self, xitroo, mailId: str):
        """
        Mail constructor
        :param xitroo: Xitroo object.
        :param mailId: Mail id to identify mail to read.
        :type xitroo: :class:`xitroo.Xitroo.Xitroo`
        :type mailId: :class:`str`
        """
        self._xitroo = xitroo
        self._mailId: str = mailId

    def getBodyHtml(self) -> str:
        """
        Get body html content of ``mailId``.
        :rtype: :class:`str`
        :return: :class:`str`
        """
        return base64.b64decode(self.getRawMail()['bodyHtml']).decode('utf-8')

    def getBodyHtmlStrict(self) -> str:
        """
        Get strict body html content of ``mailId``.
        :rtype: :class:`str`
        :return: :class:`str`
        """
        return base64.b64decode(self.getRawMail()['bodyHtmlStrict']).decode('utf-8')

    def getBodyText(self) -> str:
        """
        Get text content of ``mailId``.
        :rtype: :class:`str`
        :return: :class:`str`
        """
        return base64.b64decode(self.getRawMail()['bodyText']).decode('utf-8')

    def getContentType(self) -> str:
        """
        Get content type of ``mailId``.
        :rtype: :class:`str`
        :return: :class:`str`
        """
        return self.getRawMail()['contentType']

    def getExpiration(self) -> float:
        """
        Get expiration timestamp of ``mailId``.
        :rtype: :class:`float`
        :return: :class:`float`
        """
        return self.getRawMail()['expireTimestamp']

    def getFromMail(self) -> str:
        """
        Get mail sender from ``mailId``.
        :rtype: :class:`str`
        :return: :class:`str`
        """
        return self.getRawMail()['from']

    def getRawHeader(self) -> str:
        """
        Get raw header content of ``mailId``.
        :rtype: :class:`str`
        :return: :class:`str`
        """
        return self.getRawMail()['rawHeader']

    def getArrivalTimestamp(self) -> float:
        """
        Get arrival timestamp of ``mailId``.
        :rtype: :class:`float`
        :return: :class:`str`
        """
        return self.getRawMail()['arrivalTimestamp']

    def compareTimeTo(self, Mailobj: __init__) -> float:
        """
        Compare timestamp of ``self`` to ``Mailobj``.
        :param Mailobj: Another Mail object to compare.
        :type Mailobj: :class:`Mail`
        :rtype: :class:`float`
        :return: :class:`float` value between ``self`` and ``Mailobj``.
        """
        return self.getArrivalTimestamp() - Mailobj.getArrivalTimestamp()

    def getSubject(self) -> str:
        """
        Get subject of ``mailId``.
        :rtype: :class:`str`
        :return: :class:`str`
        """
        return base64.b64decode(self.getRawMail()['subject']).decode('utf-8')

    def getAttachments(self) -> list:
        """
        Get attachments of ``mailId``.
        :rtype: :class:`list`
        :return: :class:`list`
        """
        return self.getRawMail()['attachments']

    def downloadAttachment(self, filetodownload: str, path: str = None) -> bool:
        """
        Download attachments of ``mailId`` to ``path``.
        :param filetodownload: Attachment to download.
        :param path: Path where to save the downloaded file.
        :type filetodownload: :class:`str`
        :type path: :class:`str`
        :return: :class:`bool` indicating success or failure.
        """
        fpath = filetodownload if path is None else os.path.join(path, filetodownload)
        if os.path.exists(fpath):
            return False
        params: dict[str, str] = {
            "locale": self._xitroo._locale,
            "filename": filetodownload,
            "id": self._mailId,
        }
        r: requests.Response = self._xitroo._session.get(self._xitroo._ATTACHMENT, params=params)
        if r.status_code != 200:
            return False
        data = r.json()["data"]
        with open(fpath, 'wb') as f:
            f.write(base64.b64decode(data))
            f.close()
        return True

    def readAttachment(self, filetoread: str, encoding: str = 'utf-8') -> str:
        """
        Read attachment of ``filetoread`` in ``mailId`` to desired ``encoding``.
        :param filetoread: Attachment to read.
        :param encoding: Encoding to use when reading the attachment.
        :type filetoread: :class:`str`
        :type encoding: :class:`str`
        :rtype: :class:`str`
        :return: :class:`str`
        """
        params: dict[str, str] = {
            "locale": self._xitroo._locale,
            "filename": filetoread,
            "id": self._mailId,
        }
        r: requests.Response = self._xitroo._session.get(self._xitroo._ATTACHMENT, params=params)
        if r.status_code != 200:
            return "Error reading attachment"
        return base64.b64decode(r.json()["data"]).decode(encoding)


    def getRawMail(self) -> dict:
        """
        Get raw mail content of ``mailId``.
        :rtype: :class:`dict`
        :return: :class:`dict` of ``mailId``
        """
        params: dict[str, str] = {
            "locale": self._xitroo._locale,
            "id": self._mailId
        }
        return self._xitroo._session.get(self._xitroo._MAIL, params=params).json()



from .exceptions import CaptchaException
class Captcha:
    def __init__(self, xitroo):
        """
        Captcha constructor.
        :param xitroo: Xitroo object.
        :type xitroo: :class:`xitroo.Xitroo.Xitroo`
        """
        self._xitroo = xitroo
        self.id: str = ""

    def getCaptcha(self) -> dict:
        """
        Get captcha data as dict.
        :return: :class:`dict`
        """
        params: dict[str, str] = {"locale": self._xitroo._locale}
        r: dict = self._xitroo._session.get(self._xitroo._GETCAPTCHA, headers=self._xitroo._header,
                                            params=params).json()
        self.id: str = r["authID"]
        return r

    def verifyCaptcha(self, solution: str, id: str = "") -> bool:
        """
        Verify the captcha by checking if the solution is correct.
        :param solution: Captcha solution.
        :param id: optional Captcha id if creating a new Captcha object.
        :type solution: :class:`str`
        :type id: :class:`str`
        :rtype: :class:`bool`
        :return: :class:`bool` indicating if the solution is correct.
        """
        if self.id is None and id:
            self.id: str = id
        if not self.id and not id:
            raise CaptchaException("Create Captcha Object or pass in a captcha id")
        params: dict[str, str] = {"locale": self._xitroo._locale,
                                  "authID": self.id,
                                  "captchaSolution": solution
                                  }
        r: dict = self._xitroo._session.get(self._xitroo._SENDCAPTCHA, headers=self._xitroo._header,
                                            params=params).json()
        if not r["authSuccess"]:
            return False
        return True



from .Xitroo import Xitroo
from .exceptions import *

__title__ = "Unofficial Xitroo API"
__description__ = "Xitroo library for Python"
__url__ = "https://github.com/Th3K1n91/xitroo_api"
__version__ = "0.8"
__author__ = "InSuckaBlyat88"
__author_email__ = "x.xxha001@gmail.com"





from datetime import datetime
from .Inbox import Inbox as Inboxclass
class SearchMail:
    # {'totalMails': 0, 'mails': []}
    def __init__(self, xitroo):
        """
        SearchMail constructor.
        :param xitroo: Xitroo object.
        :type xitroo: :class:`xitroo.Xitroo.Xitroo`
        """
        self._xitroo = xitroo
        self._mails: dict = self._xitroo.getRawInbox()
        self._result: dict = {'totalMails': 0, 'mails': []}

    def byDate(self, mailDate: str) -> Inboxclass:
        """
        SearchMail by date.
        :param mailDate: mail date as %Y-%m-%d.
        :type mailDate: :class:`str`
        :return: :class:`Inboxclass`
        """
        # datetime.strptime(mailDate, "%Y-%m-%d").date()
        for i in self._mails['mails']:
            id: str = i['_id']
            if datetime.fromtimestamp(i['arrivalTimestamp']).strftime('%Y-%m-%d') == mailDate:
                for j in self._result['mails']:
                    if id in j['_id']:
                        return self._xitroo.Inbox(inbox=self._result)
                self._result['totalMails'] += 1
                self._result['mails'].append(i)
        return self._xitroo.Inbox(inbox=self._result)

    def bySender(self, sender: str) -> Inboxclass:
        """
        SearchMail in sender.
        :param sender: sender as string.
        :type sender: :class:`str`
        :return: :class:`Inboxclass`
        """
        for i in self._mails['mails']:
            id: str = i['_id']
            if sender in self._xitroo.Mail(id).getFromMail():
                for j in self._result['mails']:
                    if id in j['_id']:
                        return self._xitroo.Inbox(inbox=self._result)
                self._result['totalMails'] += 1
                self._result['mails'].append(i)
        return self._xitroo.Inbox(inbox=self._result)

    def byTitle(self, title: str) -> Inboxclass:
        """
        SearchMail in title.
        :param title: title as string.
        :type title: :class:`str`
        :return: :class:`Inboxclass`
        """
        for i in self._mails['mails']:
            id: str = i['_id']
            if title in self._xitroo.Mail(id).getSubject():
                for j in self._result['mails']:
                    if id in j['_id']:
                        return self._xitroo.Inbox(inbox=self._result)
                self._result['totalMails'] += 1
                self._result['mails'].append(i)
        return self._xitroo.Inbox(inbox=self._result)

    def byTextInBody(self, text: str) -> Inboxclass:
        """
        SearchMail in text body.
        :param text: text as string.
        :type text: :class:`str`
        :return: :class:`Inboxclass`
        """
        for i in self._mails['mails']:
            id: str = i['_id']
            if text in self._xitroo.Mail(id).getBodyText():
                for j in self._result['mails']:
                    if id in j['_id']:
                        return self._xitroo.Inbox(inbox=self._result)
                self._result['totalMails'] += 1
                self._result['mails'].append(i)
        return self._xitroo.Inbox(inbox=self._result)

    # def searchInboxRegex(self):
    #     # {'totalMails': 0, 'mails': []}
    #     class SearchRe:
    #         def __init__(self, xitroo: Xitroo):
    #
    #         def byDate(self, regex: str):
    #                 if re.search(regex, datetime.fromtimestamp(i['arrivalTimestamp']).strftime('%Y-%m-%d')):
    #
    #         def bySender(self, regex: str):
    #                 if re.search(regex, self._xitroo.Mail(id).getFromMail()):
    #
    #         def byTitle(self, regex: str):
    #                 if re.search(regex, self._xitroo.Mail(id).getSubject()):
    #
    #         def byTextInBody(self, regex: str):
    #                 if re.search(regex, self._xitroo.Mail(id).getBodyText()):
    #     return SearchRe(self)
